package com.nisum.webfluxmongodbordermanagement.config;

import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

@Component
@Slf4j
public class KafkaProducer {

    private final KafkaTemplate<String, String> kafkaTemplate;

    @Autowired
    public KafkaProducer(KafkaTemplate<String, String> kafkaTemplate) {
        this.kafkaTemplate = kafkaTemplate;
    }

    public void sendMessage(String topic, String partitionKey, String message) {
        kafkaTemplate.send(topic, partitionKey, message)
                .addCallback(result -> {
                            RecordMetadata record = result.getRecordMetadata();
                  log.info("record sent to topic {}, partition {}, offset {}, partitionkey {}, record {}",
                          record.topic(), record.partition(), record.offset(), partitionKey, message);
                },
                        ex -> log.error("failed to send message to topic {}, partitionkey {}, message {}",
                                topic, partitionKey, message)
                );
    }

   }
