package com.nisum.webfluxmongodbordermanagement.service;

import com.nisum.webfluxmongodbordermanagement.config.KafkaProducer;
import com.nisum.webfluxmongodbordermanagement.entity.Order;
import com.nisum.webfluxmongodbordermanagement.repository.OrderRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;

import java.util.List;

@Service
@Slf4j
public class OrdersService {

    @Autowired
    private OrderRepository orderRepository;

    @Autowired
    private KafkaProducer kafkaProducer;


    @Value("${order_topic}")
    private String topicName;

    public Flux<Order> fetchOrdersAndPublish(){

        Flux<Order> orders =   orderRepository.findAll();

        Mono<List<Order>> ordersMono  = orders.collectList()
                .subscribeOn(Schedulers.boundedElastic());
        ordersMono.subscribe(
                resultList -> {
                    log.info("sending orders to kafka");
                    for (Order order : resultList) {
                        kafkaProducer.sendMessage( topicName, "order-"+order.getOrderId(), order.toString());
                    }
                },
                throwable -> log.error("Error occurred: {}",  throwable.getMessage())
        );
        return  orders;
    }

}
