package com.nisum.webfluxmongodbordermanagement.service;

import com.nisum.webfluxmongodbordermanagement.config.KafkaProducer;
import com.nisum.webfluxmongodbordermanagement.entity.UserOrders;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.aggregation.LookupOperation;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;

import java.util.List;


@Service
@Slf4j
public class UserOrdersService {

    @Autowired
     ReactiveMongoTemplate reactiveMongoTemplate;


    @Autowired
    private KafkaProducer kafkaProducer;


    @Value("${order_topic}")
    private String topicName;

    public Flux<UserOrders> getAllUsersWithOrders() {

        LookupOperation lookupOperation = LookupOperation.newLookup().
                from("orders").
                localField("userId").
                foreignField("userId").
                as("order");
        Aggregation aggregation = Aggregation.newAggregation(lookupOperation);
        return reactiveMongoTemplate.aggregate(aggregation, "user", UserOrders.class);
    }

    public Flux<UserOrders> getUsersWithOrders(String userId) {
        LookupOperation lookupOperation = LookupOperation.newLookup().
                from("orders").
                localField("userId").
                foreignField("userId").
                as("order");

        AggregationOperation match = Aggregation.match(Criteria.where("userId").is(userId));

        Aggregation aggregation = Aggregation.newAggregation(lookupOperation, match);


        return reactiveMongoTemplate.aggregate(aggregation, "user", UserOrders.class);

    }

    public void fetchUserOrdersAndPublish( Flux<UserOrders> userOrdersFlux){

        Mono<List<UserOrders>> usersMono  = userOrdersFlux.collectList()
                .subscribeOn(Schedulers.boundedElastic());
        usersMono
                .subscribe(
                        resultList -> {
                            log.info("sending userOrders to kafka");
                            for (UserOrders user : resultList) {
                                kafkaProducer.sendMessage( topicName, "userOrders-"+user.getUserId(), user.toString());
                            }
                        },
                        throwable -> log.error("Error occurred: {}",  throwable.getMessage())
                );
    }

}



















//    @Autowired
//    // private MongoTemplate mongoTemplate;
//    ReactiveMongoTemplate mongoTemplate;
//
//    @Autowired
//    UserRepository userRepository;
//
//    public UserService(ReactiveMongoTemplate mongoTemplate) {
//        this.mongoTemplate = mongoTemplate;
//    }
//
//
//    public Mono<User> addUser(User user){
//
//        return userRepository.save(user);
//    }
//
//    public Flux<?> aggregateUserOrders(String userId) {
//        LookupOperation lookupOperation = LookupOperation.newLookup()
//                .from("Orders")
//                .localField("userId")
//                .foreignField("userId")
//                .as("orders");
//
//
////        Aggregation aggregation = Aggregation.newAggregation(lookupOperation, Aggregation.match(Criteria.where("userId").is(userId)));
////        //Flux<Document> results = mongoTemplate.aggregate(aggregation, "User", Document.class);
//        //Set<UserOrders> users = results.toStream().collect(Collectors.toSet());
//        //List<UserOrders> result = results.collectList().block();
//        //log.info("results {} ", result);
////        Flux<AggregationResults> results = mongoTemplate.aggregate(aggregation, "User", AggregationResults.class)
////                .map(AggregationResults::getMappedResults);
////                //.flatMap(1);
//        //.flatMapIterable(Flux::fromIterable);
////        //Iterable<Document> orderedTests = (List<Document>) results.collectList().block().getRawResults().get(“results”);
////        log.info("Obj Size " +results);
////
////        return results;
////    }



    /*private final ReactiveMongoTemplate mongoTemplate;

    public UserService(ReactiveMongoTemplate mongoTemplate) {
        this.mongoTemplate = mongoTemplate;
    }

    @Autowired
    UserRepository userRepository;

    @Autowired
    OrderRepository orderRepository;

    public Mono<User> addUser(User user){

        return userRepository.save(user);
    }



    public Flux<User> getUsersWithOrders() {
        return userRepository.findAll();


//        Aggregation aggregation = Aggregation.newAggregation(
//                Aggregation.match(Criteria.where("userId").is(userRepository)),
//                Aggregation.lookup("Orders", "userId", "_Id", "Orders"),
//                Aggregation.unwind("Orders", true)
//        );
       // Flux<User> users = mongoTemplate.aggregate(aggregation, "users", User.class);

        //db.User.aggregate([{$lookup:{from:"Orders", localField:"userId",foreignField:"_id",as:"Orders"}}])

    }

     */


