package com.nisum.webfluxmongodbordermanagement.service;


import com.nisum.webfluxmongodbordermanagement.config.KafkaProducer;
import com.nisum.webfluxmongodbordermanagement.entity.User;
import com.nisum.webfluxmongodbordermanagement.repository.UserRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;

import java.util.List;

@Service
@Slf4j
public class UserService {

    @Autowired
    private UserRepository userRepository;


@Autowired
private KafkaProducer kafkaProducer;


    @Value("${order_topic}")
    private String topicName;

//    public Flux<User> getUsers(){
//
//        return userRepository.findAll();
//    }


    public Flux<User> fetchUsersAndPublish(){

        Flux<User> users =   userRepository.findAll();

        Mono<List<User>> usersMono  = users.collectList()
                .subscribeOn(Schedulers.boundedElastic());
        usersMono
                .subscribe(
                resultList -> {
                    log.info("sending users to kafka");
                    for (User user : resultList) {
                        kafkaProducer.sendMessage( topicName, "user-"+user.getUserId(), user.toString());
                    }
                },
                throwable -> log.error("Error occurred: {}",  throwable.getMessage())
        );
        return users;
    }

}