package com.nisum.webfluxmongodbordermanagement.service;

import com.nisum.webfluxmongodbordermanagement.config.KafkaProducer;
import com.nisum.webfluxmongodbordermanagement.entity.Order;
import com.nisum.webfluxmongodbordermanagement.entity.User;
import com.nisum.webfluxmongodbordermanagement.repository.OrderRepository;
import com.nisum.webfluxmongodbordermanagement.repository.UserRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import reactor.core.publisher.Flux;
import reactor.test.StepVerifier;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;

public class OrdersServiceTest {
    @Autowired
    private ReactiveMongoTemplate reactiveMongoTemplate;

    @Mock
    OrderRepository orderRepository;

    @InjectMocks
   OrdersService ordersService;
    @Mock
    KafkaProducer kafkaProducer;


    @BeforeEach
    void setUp() {
        MockitoAnnotations.initMocks(this);
    }


    @Test
    @DisplayName("get all orders ")
    void getAllOrders() {
       Order order1 = new Order("1123v34","1", "1","oppo123445", "oppo123", "success");
        Order order2 = new Order("112345","2", "1","vivo123445", "vivo123", "success");
        when(orderRepository.findAll()).thenReturn(Flux.just(order1, order2));
        doNothing().when(kafkaProducer).sendMessage(anyString(),anyString(),anyString());
        // Call the method
        Flux<Order> result = ordersService.fetchOrdersAndPublish();

        // Verify the output
        StepVerifier.create(result)
                .expectNext(order1)
                .expectNext(order2)
                .expectComplete()
                .verify();

    }
}


