package com.nisum.webfluxmongodbordermanagement.service;

import com.nisum.webfluxmongodbordermanagement.config.KafkaProducer;
import com.nisum.webfluxmongodbordermanagement.entity.User;
import com.nisum.webfluxmongodbordermanagement.repository.UserRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import reactor.core.publisher.Flux;
import reactor.test.StepVerifier;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;

class UserServiceTest {
    @Autowired
    private ReactiveMongoTemplate reactiveMongoTemplate;

    @Mock
    UserRepository userRepository;

    @InjectMocks
    UserService userService;

    @Mock
    KafkaProducer kafkaProducer;



    @BeforeEach
    void setUp() {
       MockitoAnnotations.initMocks(this);
    }


    @Test
    @DisplayName("get all users ")
    void getAllUsers() {
        User user1 = new User("1","01", "John", "john@example.com");
        User user2 = new User("2","02","Jane", "jane@example.com");
        when(userRepository.findAll()).thenReturn(Flux.just(user1, user2));
        doNothing().when(kafkaProducer).sendMessage(anyString(),anyString(),anyString());

        // Call the method
        Flux<User> result = userService.fetchUsersAndPublish();

        // Verify the output
        StepVerifier.create(result)
                .expectNext(user1)
                .expectNext(user2)
                .expectComplete()
                .verify();

    }
    }
